'''test_cr_online_accounts.py: tests for the cr_online accounts module'''
#
# Copyright (C) 2013 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from clickreviews.cr_online_accounts import ClickReviewAccounts
import clickreviews.cr_tests as cr_tests
import json
import lxml.etree as etree


class TestClickReviewAccounts(cr_tests.TestClickReview):
    """Tests for the lint review tool."""

    def _stub_application(self, root=None, id=None, do_subtree=True):
        '''Stub application xml'''
        if root is None:
            root = "application"
        if id == "" or id is None:
            xml = etree.Element(root)
        else:
            xml = etree.Element(root, id="%s" % id)
        if do_subtree:
            services = etree.SubElement(xml, "services")
            if id is None:
                elem1 = etree.SubElement(services, "service")
            else:
                elem1 = etree.SubElement(services, "service", id="element1")
            desc1 = etree.SubElement(elem1, "description")
            desc1.text = "elem1 description"
            if id is None:
                elem2 = etree.SubElement(services, "service")
            else:
                elem2 = etree.SubElement(services, "service", id="element2")
            desc2 = etree.SubElement(elem2, "description")
            desc2.text = "elem2 description"
        return xml

    def _stub_service(self, root=None, id=None, do_subtree=True):
        '''Stub service xml'''
        if root is None:
            root = "service"
        if id == "" or id is None:
            xml = etree.Element(root)
        else:
            xml = etree.Element(root, id="%s" % id)
        if do_subtree:
            service_name = etree.SubElement(xml, "name")
            service_name.text = "Foo"
            service_provider = etree.SubElement(xml, "provider")
            service_provider.text = "some-provider"
        return xml

    def _stub_provider(self, root=None, id=None, do_subtree=True):
        '''Stub provider xml'''
        if root is None:
            root = "provider"
        if id == "" or id is None:
            xml = etree.Element(root)
        else:
            xml = etree.Element(root, id="%s" % id)
        if do_subtree:
            service_name = etree.SubElement(xml, "name")
            service_name.text = "Foo"
            service_plugin = etree.SubElement(xml, "plugin")
            service_plugin.text = "generic-oauth"
            service_domains = etree.SubElement(xml, "domains")
            service_domains.text = r".*\.example\.com"
            # More can go here, see /usr/share/accounts/providers/*
        return xml

    def test_check_hooks_versions_new(self):
        '''Test check_hooks_versions() - new hook'''
        self.set_test_manifest("framework", "ubuntu-sdk-15.04.1")
        self.set_test_account(self.default_appname, "accounts", dict())
        c = ClickReviewAccounts(self.test_name)
        c.check_hooks_versions()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_hooks_versions_deprecated_service(self):
        '''Test check_hooks_versions() - deprecated -service hook'''
        self.set_test_manifest("framework", "ubuntu-sdk-15.10")
        self.set_test_account(self.default_appname, "account-service", dict())
        c = ClickReviewAccounts(self.test_name)
        c.check_hooks_versions()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_hooks_versions_disallowed_service(self):
        '''Test check_hooks_versions() - deprecated -service hook'''
        self.set_test_manifest("framework", "ubuntu-sdk-16.10")
        self.set_test_account(self.default_appname, "account-service", dict())
        c = ClickReviewAccounts(self.test_name)
        c.check_hooks_versions()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_hooks_versions_deprecated_application(self):
        '''Test check_hooks_versions() - deprecated -application hook'''
        self.set_test_manifest("framework", "ubuntu-sdk-15.10")
        self.set_test_account(self.default_appname,
                              "account-application", dict())
        c = ClickReviewAccounts(self.test_name)
        c.check_hooks_versions()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_hooks_versions_old_framework(self):
        '''Test check_hooks_versions() - deprecated -application hook'''
        self.set_test_manifest("framework", "ubuntu-sdk-15.04")
        self.set_test_account(self.default_appname,
                              "account-application", dict())
        self.set_test_account(self.default_appname, "account-service", dict())
        c = ClickReviewAccounts(self.test_name)
        c.check_hooks_versions()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_manifest(self):
        '''Test check_manifest()'''
        data = json.loads('''{
          "translations": "my-app",
          "services": [
            {
              "name": "Example",
              "provider": "myapp.com_example",
              "description": "publish my photos in example.com",
              "auth": {
                "oauth2/web_server": {
                  "ClientId": "foo",
                  "ClientSecret": "bar",
                  "UseSSL": false,
                  "Scopes": ["one scope","and another"]
                }
              }
            },
            {
              "provider": "becool"
            }
          ],
          "plugin": {
              "name": "Example site",
              "icon": "example.png",
              "qml": "qml_files"
          }
        }''')
        self.set_test_account(self.default_appname, "accounts", data)
        c = ClickReviewAccounts(self.test_name)
        c.check_manifest()
        r = c.click_report
        expected_counts = {'info': 3, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_manifest_not_specified(self):
        '''Test check_manifest() - not specified'''
        c = ClickReviewAccounts(self.test_name)
        c.check_manifest()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_manifest_missing_services(self):
        '''Test check_manifest() - missing services'''
        data = json.loads('''{ "translations": "my-app" }''')
        self.set_test_account(self.default_appname, "accounts", data)
        c = ClickReviewAccounts(self.test_name)
        c.check_manifest()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_manifest_invalid_services(self):
        '''Test check_manifest() - invalid services'''
        data = json.loads('''{ "services": 12 }''')
        self.set_test_account(self.default_appname, "accounts", data)
        c = ClickReviewAccounts(self.test_name)
        c.check_manifest()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_manifest_empty_services(self):
        '''Test check_manifest() - empty services'''
        data = json.loads('''{ "services": [] }''')
        self.set_test_account(self.default_appname, "accounts", data)
        c = ClickReviewAccounts(self.test_name)
        c.check_manifest()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_manifest_empty_service(self):
        '''Test check_manifest() - empty services'''
        data = json.loads('''{ "services": [{}] }''')
        self.set_test_account(self.default_appname, "accounts", data)
        c = ClickReviewAccounts(self.test_name)
        c.check_manifest()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_manifest_no_provider(self):
        '''Test check_manifest() - no provider'''
        data = json.loads('''{ "services": [{
          "name": "Example",
          "description": "Hello world"
        }] }''')
        self.set_test_account(self.default_appname, "accounts", data)
        c = ClickReviewAccounts(self.test_name)
        c.check_manifest()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_manifest_invalid_provider(self):
        '''Test check_manifest() - invalid provider'''
        data = json.loads('''{ "services": [{
          "name": "Example",
          "provider": "no/slashes.please",
          "description": "Hello world"
        }] }''')
        self.set_test_account(self.default_appname, "accounts", data)
        c = ClickReviewAccounts(self.test_name)
        c.check_manifest()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_manifest_unknown_key(self):
        '''Test check_manifest() - unknown key'''
        data = json.loads('''{ "services": [{
          "name": "Example",
          "provider": "example",
          "description": "Hello world",
          "intruder": "Who, me?"
        }] }''')
        self.set_test_account(self.default_appname, "accounts", data)
        c = ClickReviewAccounts(self.test_name)
        c.check_manifest()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_application(self):
        '''Test check_application()'''
        xml = self._stub_application()
        # print(etree.tostring(xml))
        self.set_test_account(self.default_appname, "account-application", xml)
        c = ClickReviewAccounts(self.test_name)
        c.check_application()
        r = c.click_report
        expected_counts = {'info': 4, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_application_snappy_1504(self):
        '''Test check_application() - snappy 15.04'''
        self.set_test_pkgfmt("snap", "15.04")
        xml = self._stub_application()
        # print(etree.tostring(xml))
        self.set_test_account(self.default_appname, "account-application", xml)
        c = ClickReviewAccounts(self.test_name)
        c.check_application()
        r = c.click_report
        expected_counts = {'info': 4, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_application_not_specified(self):
        '''Test check_application() - not specified'''
        c = ClickReviewAccounts(self.test_name)
        c.check_application()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_application_has_id(self):
        '''Test check_application() - has id'''
        xml = self._stub_application(id="%s_%s" % (self.test_manifest["name"],
                                                   self.default_appname))
        self.set_test_account(self.default_appname, "account-application", xml)
        c = ClickReviewAccounts(self.test_name)
        c.check_application()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_application_wrong_root(self):
        '''Test check_application() - wrong root'''
        xml = self._stub_application(root="wrongroot")
        self.set_test_account(self.default_appname, "account-application", xml)
        c = ClickReviewAccounts(self.test_name)
        c.check_application()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_application_missing_services(self):
        '''Test check_application() - missing services'''
        xml = self._stub_application(do_subtree=False)

        sometag = etree.SubElement(xml, "sometag")
        elem1 = etree.SubElement(sometag, "something", id="element1")
        desc1 = etree.SubElement(elem1, "description")
        desc1.text = "elem1 description"

        self.set_test_account(self.default_appname, "account-application", xml)
        c = ClickReviewAccounts(self.test_name)
        c.check_application()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_application_missing_service(self):
        '''Test check_application() - missing service'''
        xml = self._stub_application(do_subtree=False)

        services = etree.SubElement(xml, "services")
        elem1 = etree.SubElement(services, "somesubtag", id="element1")
        desc1 = etree.SubElement(elem1, "description")
        desc1.text = "elem1 description"

        self.set_test_account(self.default_appname, "account-application", xml)
        c = ClickReviewAccounts(self.test_name)
        c.check_application()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_service(self):
        '''Test check_service()'''
        xml = self._stub_service()
        self.set_test_account(self.default_appname, "account-service", xml)
        xml = self._stub_application()
        self.set_test_account(self.default_appname, "account-application", xml)
        c = ClickReviewAccounts(self.test_name)
        c.check_service()
        r = c.click_report
        expected_counts = {'info': 4, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_service_not_specified(self):
        '''Test check_service() - not specified'''
        c = ClickReviewAccounts(self.test_name)
        c.check_service()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_service_has_id(self):
        '''Test check_service() - has id'''
        xml = self._stub_service(id="%s_%s" % (self.test_manifest["name"],
                                               self.default_appname))
        self.set_test_account(self.default_appname, "account-service", xml)
        xml = self._stub_application()
        self.set_test_account(self.default_appname, "account-application", xml)
        c = ClickReviewAccounts(self.test_name)
        c.check_service()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_service_wrong_root(self):
        '''Test check_service() - wrong root'''
        xml = self._stub_service(root="wrongroot")
        self.set_test_account(self.default_appname, "account-service", xml)
        xml = self._stub_application()
        self.set_test_account(self.default_appname, "account-application", xml)
        c = ClickReviewAccounts(self.test_name)
        c.check_service()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_service_missing_name(self):
        '''Test check_service() - missing name'''
        xml = self._stub_service(do_subtree=False)
        service_provider = etree.SubElement(xml, "provider")
        service_provider.text = "some-provider"
        self.set_test_account(self.default_appname, "account-service", xml)
        xml = self._stub_application()
        self.set_test_account(self.default_appname, "account-application", xml)
        c = ClickReviewAccounts(self.test_name)
        c.check_service()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_service_missing_provider(self):
        '''Test check_service() - missing provider'''
        xml = self._stub_service(do_subtree=False)
        service_name = etree.SubElement(xml, "name")
        service_name.text = "Foo"
        self.set_test_account(self.default_appname, "account-service", xml)
        xml = self._stub_application()
        self.set_test_account(self.default_appname, "account-application", xml)
        c = ClickReviewAccounts(self.test_name)
        c.check_service()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_provider(self):
        '''Test check_provider()'''
        xml = self._stub_provider()
        self.set_test_account(self.default_appname, "account-provider", xml)
        self.set_test_account(self.default_appname, "account-qml-plugin", True)
        c = ClickReviewAccounts(self.test_name)
        c.check_provider()
        r = c.click_report
        expected_counts = {'info': 3, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_provider_has_id(self):
        '''Test check_provider() - has id'''
        xml = self._stub_provider(id="%s_%s" % (self.test_manifest["name"],
                                                self.default_appname))
        self.set_test_account(self.default_appname, "account-provider", xml)
        self.set_test_account(self.default_appname, "account-qml-plugin", True)
        c = ClickReviewAccounts(self.test_name)
        c.check_provider()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_qml_plugin(self):
        '''Test check_qml_plugin()'''
        self.set_test_account(self.default_appname, "account-qml-plugin", True)
        xml = self._stub_provider()
        self.set_test_account(self.default_appname, "account-provider", xml)
        c = ClickReviewAccounts(self.test_name)
        c.check_qml_plugin()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_peer_hooks_application(self):
        '''Test check_peer_hooks() - application'''
        c = ClickReviewAccounts(self.test_name)

        # create a new hooks database for our peer hooks tests
        tmp = dict()

        # add our hook
        tmp["account-application"] = "foo.application"

        # add any required peer hooks
        tmp["apparmor"] = "foo.apparmor"

        # update the manifest and test_manifest
        c.manifest["hooks"][self.default_appname] = tmp
        self._update_test_manifest()

        # do the test
        c.check_peer_hooks(["account-application"])
        r = c.click_report
        # We should end up with 8 info
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_peer_hooks_application_disallowed(self):
        '''Test check_peer_hooks() - disallowed (application)'''
        c = ClickReviewAccounts(self.test_name)

        # create a new hooks database for our peer hooks tests
        tmp = dict()

        # add our hook
        tmp["account-application"] = "foo.application"

        # add any required peer hooks
        tmp["apparmor"] = "foo.apparmor"

        # add something not allowed
        tmp["nonexistent"] = "nonexistent-hook"

        c.manifest["hooks"][self.default_appname] = tmp
        self._update_test_manifest()

        # do the test
        c.check_peer_hooks(["account-application"])
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_peer_hooks_application_required(self):
        '''Test check_peer_hooks() - required (application)'''
        c = ClickReviewAccounts(self.test_name)

        # create a new hooks database for our peer hooks tests
        tmp = dict()

        # add our hook
        tmp["account-application"] = "foo.application"

        # skip adding required hooks

        c.manifest["hooks"][self.default_appname] = tmp
        self._update_test_manifest()

        # do the test
        c.check_peer_hooks(["account-application"])
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_peer_hooks_service(self):
        '''Test check_peer_hooks() - service'''
        c = ClickReviewAccounts(self.test_name)

        # create a new hooks database for our peer hooks tests
        tmp = dict()

        # add our hook
        tmp["account-service"] = "foo.service"

        # add any required peer hooks
        tmp["account-application"] = "foo.application"
        tmp["apparmor"] = "foo.apparmor"

        # update the manifest and test_manifest
        c.manifest["hooks"][self.default_appname] = tmp
        self._update_test_manifest()

        # do the test
        c.check_peer_hooks(["account-service"])
        r = c.click_report
        # We should end up with 8 info
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_peer_hooks_service_disallowed(self):
        '''Test check_peer_hooks() - disallowed (service)'''
        c = ClickReviewAccounts(self.test_name)

        # create a new hooks database for our peer hooks tests
        tmp = dict()

        # add our hook
        tmp["account-service"] = "foo.service"

        # add any required peer hooks
        tmp["account-application"] = "foo.application"
        tmp["apparmor"] = "foo.apparmor"

        # add something not allowed
        tmp["nonexistent"] = "nonexistent-hook"

        c.manifest["hooks"][self.default_appname] = tmp
        self._update_test_manifest()

        # do the test
        c.check_peer_hooks(["account-service"])
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_peer_hooks_service_required(self):
        '''Test check_peer_hooks() - required (service)'''
        c = ClickReviewAccounts(self.test_name)

        # create a new hooks database for our peer hooks tests
        tmp = dict()

        # add our hook
        tmp["account-service"] = "foo.service"

        # skip adding required hooks

        c.manifest["hooks"][self.default_appname] = tmp
        self._update_test_manifest()

        # do the test
        c.check_peer_hooks(["account-service"])
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_peer_hooks_provider(self):
        '''Test check_peer_hooks() - provider'''
        c = ClickReviewAccounts(self.test_name)

        # create a new hooks database for our peer hooks tests
        tmp = dict()

        # add our hook
        tmp["account-provider"] = "foo.provider"

        # add any required peer hooks
        tmp["account-qml-plugin"] = "foo.qml_plugin"
        tmp["apparmor"] = "foo.apparmor"

        # update the manifest and test_manifest
        c.manifest["hooks"][self.default_appname] = tmp
        self._update_test_manifest()

        # do the test
        c.check_peer_hooks(["account-provider"])
        r = c.click_report
        # We should end up with 8 info
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_check_peer_hooks_provider_disallowed(self):
        '''Test check_peer_hooks() - disallowed (provider)'''
        c = ClickReviewAccounts(self.test_name)

        # create a new hooks database for our peer hooks tests
        tmp = dict()

        # add our hook
        tmp["account-provider"] = "foo.provider"

        # add any required peer hooks
        tmp["account-qml-plugin"] = "foo.qml_plugin"
        tmp["apparmor"] = "foo.apparmor"

        # add something not allowed
        tmp["nonexistent"] = "nonexistent-hook"

        c.manifest["hooks"][self.default_appname] = tmp
        self._update_test_manifest()

        # do the test
        c.check_peer_hooks(["account-provider"])
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_check_peer_hooks_provider_required(self):
        '''Test check_peer_hooks() - required (provider)'''
        c = ClickReviewAccounts(self.test_name)

        # create a new hooks database for our peer hooks tests
        tmp = dict()

        # add our hook
        tmp["account-provider"] = "foo.provider"

        # skip adding required hooks

        c.manifest["hooks"][self.default_appname] = tmp
        self._update_test_manifest()

        # do the test
        c.check_peer_hooks(["account-provider"])
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)



'''cr_online_accounts.py: click online accounts'''
#
# Copyright (C) 2013-2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function

from clickreviews.cr_common import ClickReview, error, open_file_read
import json
import os
import re
# http://lxml.de/tutorial.html
import lxml.etree as etree


class ClickReviewAccounts(ClickReview):
    '''This class represents click lint reviews'''
    def __init__(self, fn, overrides=None):
        peer_hooks = dict()
        peer_hooks['account-application'] = dict()
        peer_hooks['account-application']['allowed'] = \
            ClickReview.app_allowed_peer_hooks + \
            ClickReview.scope_allowed_peer_hooks
        peer_hooks['account-application']['required'] = ['apparmor']

        peer_hooks['account-service'] = dict()
        peer_hooks['account-service']['required'] = ['account-application',
                                                     'apparmor'
                                                     ]
        peer_hooks['account-service']['allowed'] = \
            ClickReview.app_allowed_peer_hooks + \
            ClickReview.scope_allowed_peer_hooks

        peer_hooks['accounts'] = dict()
        peer_hooks['accounts']['allowed'] = \
            [h for h in (ClickReview.app_allowed_peer_hooks +
                         ClickReview.scope_allowed_peer_hooks)
             if not h.startswith('account-')]
        peer_hooks['accounts']['required'] = ['apparmor']

        peer_hooks['account-provider'] = dict()
        peer_hooks['account-provider']['required'] = ['account-qml-plugin',
                                                      'apparmor'
                                                      ]
        peer_hooks['account-provider']['allowed'] = \
            peer_hooks['account-provider']['required']

        peer_hooks['account-qml-plugin'] = dict()
        peer_hooks['account-qml-plugin']['required'] = ['account-provider',
                                                        'apparmor'
                                                        ]
        peer_hooks['account-qml-plugin']['allowed'] = \
            peer_hooks['account-qml-plugin']['required']

        link = ("https://wiki.ubuntu.com/SecurityTeam/Specifications/"
                "OnlineAccountsConfinement")
        ClickReview.__init__(self,
                             fn,
                             "online_accounts",
                             peer_hooks=peer_hooks,
                             overrides=overrides,
                             peer_hooks_link=link)
        if not self.is_click and not self.is_snap1:
            return

        self.accounts_files = dict()
        self.accounts = dict()

        self.account_hooks = ['accounts',
                              'account-application',
                              'account-provider',
                              'account-qml-plugin',
                              'account-service']

        if self.manifest is None:
            return

        for app in self.manifest['hooks']:
            for h in self.account_hooks:
                if h not in self.manifest['hooks'][app]:
                    # msg("Skipped missing %s hook for '%s'" % (h, app))
                    continue
                if not isinstance(self.manifest['hooks'][app][h], str):
                    error("manifest malformed: hooks/%s/%s is not a str" % (
                          app, h))

                (full_fn, parsed) = self._extract_account(app, h)

                if app not in self.accounts_files:
                    self.accounts_files[app] = dict()
                self.accounts_files[app][h] = full_fn

                if app not in self.accounts:
                    self.accounts[app] = dict()
                self.accounts[app][h] = parsed

        self.required_keys = dict()
        self.allowed_keys = dict()
        self.required_keys["service"] = [
            ('provider', str),
        ]
        self.allowed_keys["service"] = [
            ('auth', dict),
            ('name', str),
            ('description', str),
        ]
        self.required_keys["plugin"] = [
            ('name', str),
            ('icon', str),
            ('qml', str),
        ]
        self.allowed_keys["plugin"] = [
            ('auth', dict),
        ]
        self.provider_re = re.compile('^[a-zA-Z0-9_.-]+$')

    def _extract_account(self, app, account_type):
        '''Extract accounts'''
        if self.manifest is None:
            return

        a = self.manifest['hooks'][app][account_type]
        fn = os.path.join(self.unpack_dir, a)

        bn = os.path.basename(fn)
        if not os.path.exists(fn):
            error("Could not find '%s'" % bn)

        # qml-plugin points to a QML file, so just set that we have the
        # the hook present for now
        if account_type == "account-qml-plugin":
            return (fn, True)
        elif account_type == "accounts":
            fh = open_file_read(fn)
            contents = ""
            for line in fh.readlines():
                contents += line
            fh.close()

            try:
                jd = json.loads(contents)
            except Exception as e:
                error("accounts json unparseable: %s (%s):\n%s" % (bn,
                      str(e), contents))

            if not isinstance(jd, dict):
                error("accounts json is malformed: %s:\n%s" % (bn, contents))

            return (fn, jd)
        else:
            try:
                tree = etree.parse(fn)
                xml = tree.getroot()
            except Exception as e:
                error("accounts xml unparseable: %s (%s)" % (bn, str(e)))
            return (fn, xml)

    def check_hooks_versions(self):
        '''Check hooks versions'''
        if not self.is_click and not self.is_snap1:
            return

        if self.manifest is None:
            return

        framework = self.manifest['framework']
        if not framework.startswith("ubuntu-sdk"):
            return
        t = "error"
        if framework < "ubuntu-sdk-15.04.1":
            for app in sorted(self.accounts.keys()):
                for hook in self.accounts[app].keys():
                    if hook == "accounts":
                        n = self._get_check_name('%s_hook' % hook, app=app)
                        s = ("'accounts' hook is not available in '%s' "
                             "(must be 15.04.1 or later)" % (framework))
                        self._add_result(t, n, s)
            return
        hook_state = "disallowed"
        if framework < "ubuntu-sdk-16.10":
            t = "warn"
            hook_state = "deprecated"
        for app in sorted(self.accounts.keys()):
            for hook in self.accounts[app].keys():
                if hook.startswith("account-"):
                    n = self._get_check_name('%s_hook' % hook, app=app)
                    s = "'%s' is %s in %s: use 'accounts' hook instead" % \
                        (hook, hook_state, framework)
                    self._add_result(t, n, s)

    def _check_object(self, obj_type, obj, n):
        t = "info"
        s = "OK"
        if not isinstance(obj, dict):
            t = "error"
            s = "%s is not an object" % obj_type
            self._add_result(t, n, s)
            return

        for (k, vt) in self.required_keys[obj_type]:
            if k not in obj.keys():
                t = "error"
                s = "required key '%s' is missing" % k
                self._add_result(t, n, s)
        if t == "error":
            return

        known_keys = self.required_keys[obj_type] + self.allowed_keys[obj_type]
        for (k, v) in obj.items():
            type_list = [kk[1] for kk in known_keys if kk[0] == k]
            if len(type_list) < 1:
                t = "error"
                s = "unrecognized key '%s'" % k
                self._add_result(t, n, s)
                continue
            if not isinstance(v, type_list[0]):
                t = "error"
                s = "value for '%s' must be of type %s" % (k, type_list[0])
                self._add_result(t, n, s)
                continue
            if k == 'provider' and not self.provider_re.match(v):
                t = "error"
                s = "'provider' must only consist of alphanumeric characters"
                self._add_result(t, n, s)
        self._add_result(t, n, s)

    def _check_object_list(self, app, key, obj_type, obj_list):
        t = 'info'
        n = self._get_check_name('accounts_%s' % key, app=app)
        if not isinstance(obj_list, list):
            t = "error"
            s = "'%s' is not a list" % key
        elif len(obj_list) < 1:
            t = "error"
            s = "'%s' is empty" % key
        if t == "error":
            self._add_result(t, n, s)
            return

        for (i, obj) in enumerate(obj_list):
            n = self._get_check_name('accounts_%s' % (obj_type), app=app,
                                     extra=str(i))
            self._check_object(obj_type, obj, n)

    def check_manifest(self):
        '''Check manifest'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.accounts.keys()):
            account_type = "accounts"

            t = 'info'
            n = self._get_check_name('%s_root' % account_type, app=app)
            s = "OK"
            if account_type not in self.accounts[app]:
                s = "OK (missing)"
                self._add_result(t, n, s)
                continue

            has_services_or_plugin = False
            n = self._get_check_name('%s_services' % account_type, app=app)
            if 'services' in self.accounts[app][account_type]:
                has_services_or_plugin = True
                services = self.accounts[app][account_type]['services']
                self._check_object_list(app, "services", "service", services)

            n = self._get_check_name('%s_plugin' % account_type, app=app)
            if 'plugin' in self.accounts[app][account_type]:
                has_services_or_plugin = True
                plugin = self.accounts[app][account_type]['plugin']
                self._check_object("plugin", plugin, n)

            if not has_services_or_plugin:
                t = "error"
                s = "either 'services' or 'plugin' key must be present"
                self._add_result(t, n, s)

    def check_application(self):
        '''Check application'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.accounts.keys()):
            account_type = "account-application"

            t = 'info'
            n = self._get_check_name('%s_root' % account_type, app=app)
            s = "OK"
            if account_type not in self.accounts[app]:
                s = "OK (missing)"
                self._add_result(t, n, s)
                continue

            root_tag = self.accounts[app][account_type].tag.lower()
            if root_tag != "application":
                t = 'error'
                s = "'%s' is not 'application'" % root_tag
            self._add_result(t, n, s)

            t = 'info'
            n = self._get_check_name('%s_id' % account_type, app=app)
            s = "OK"
            if "id" in self.accounts[app][account_type].keys():
                t = 'warn'
                s = "Found 'id' in application tag"
            self._add_result(t, n, s)

            t = 'info'
            n = self._get_check_name('%s_services' % account_type, app=app)
            s = "OK"
            if self.accounts[app][account_type].find("services") is None:
                t = 'error'
                s = "Could not find '<services>' tag"
            self._add_result(t, n, s)

            if t == 'error':
                continue

            t = 'info'
            n = self._get_check_name('%s_service' % account_type, app=app)
            s = "OK"
            if self.accounts[app][account_type].find("./services/service") \
               is None:
                t = 'error'
                s = "Could not find '<service>' tag under <services>"
            self._add_result(t, n, s)

    def check_service(self):
        '''Check service'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.accounts.keys()):
            account_type = "account-service"

            t = 'info'
            n = self._get_check_name('%s_root' % account_type, app=app)
            s = "OK"
            if account_type not in self.accounts[app]:
                s = "OK (missing)"
                self._add_result(t, n, s)
                continue

            root_tag = self.accounts[app][account_type].tag.lower()
            if root_tag != "service":
                t = 'error'
                s = "'%s' is not 'service'" % root_tag
            self._add_result(t, n, s)

            t = 'info'
            n = self._get_check_name('%s_id' % account_type, app=app)
            s = "OK"
            if "id" in self.accounts[app][account_type].keys():
                t = 'warn'
                s = "Found 'id' in service tag"
            self._add_result(t, n, s)

            if t == 'error':
                continue

            link = 'http://askubuntu.com/q/697173'
            for tag in ['name', 'provider']:
                t = 'info'
                n = self._get_check_name(
                    '%s_%s' % (account_type, tag), app=app)
                s = "OK"
                if self.accounts[app][account_type].find(tag) is None:
                    t = 'error'
                    s = "Could not find '<%s>' tag" % tag
                self._add_result(t, n, s, link)

    def check_provider(self):
        '''Check provider'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.accounts.keys()):
            account_type = "account-provider"

            t = 'info'
            n = self._get_check_name(account_type, app=app)
            s = "OK"
            manual_review = False
            if account_type not in self.accounts[app]:
                s = "OK (missing)"
                self._add_result(t, n, s)
                continue
            self._add_result(t, n, s, manual_review=manual_review)

            t = 'info'
            n = self._get_check_name('%s_root' % account_type, app=app)
            s = "OK"
            root_tag = self.accounts[app][account_type].tag.lower()
            if root_tag != "provider":
                t = 'error'
                s = "'%s' is not 'provider'" % root_tag
            self._add_result(t, n, s)

            t = 'info'
            n = self._get_check_name('%s_id' % account_type, app=app)
            s = "OK"
            if "id" in self.accounts[app][account_type].keys():
                t = 'warn'
                s = "Found 'id' in provider tag"
            self._add_result(t, n, s)

    def check_qml_plugin(self):
        '''Check qml-plugin'''
        if not self.is_click and not self.is_snap1:
            return

        for app in sorted(self.accounts.keys()):
            account_type = "account-qml-plugin"

            t = 'info'
            n = self._get_check_name(account_type, app=app)
            s = "OK"
            manual_review = False
            if account_type not in self.accounts[app]:
                s = "OK (missing)"
                self._add_result(t, n, s)
                continue
            self._add_result(t, n, s, manual_review=manual_review)



#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; version 3 of the License.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

APPARMOR_POLICIES = {
    "ubuntu": {
        "1.0": {
            "templates": {
                "common": [
                    "default",
                    "ubuntu-sdk",
                    "ubuntu-webapp"
                ],
                "reserved": [
                    "unconfined"
                ]
            },
            "policy_groups": {
                "common": [
                    "audio",
                    "camera",
                    "connectivity",
                    "content_exchange",
                    "content_exchange_source",
                    "keep-display-on",
                    "location",
                    "microphone",
                    "networking",
                    "sensors",
                    "usermetrics",
                    "video"
                ],
                "reserved": [
                    "accounts",
                    "calendar",
                    "contacts",
                    "friends",
                    "history",
                    "music_files",
                    "music_files_read",
                    "picture_files",
                    "picture_files_read",
                    "video_files",
                    "video_files_read"
                ]
            }
        },
        "1.1": {
            "templates": {
                "common": [
                    "default",
                    "ubuntu-sdk",
                    "ubuntu-webapp"
                ],
                "reserved": [
                    "unconfined"
                ]
            },
            "policy_groups": {
                "common": [
                    "audio",
                    "camera",
                    "connectivity",
                    "content_exchange",
                    "content_exchange_source",
                    "keep-display-on",
                    "location",
                    "microphone",
                    "networking",
                    "sensors",
                    "usermetrics",
                    "video",
                    "webview"
                ],
                "reserved": [
                    "accounts",
                    "calendar",
                    "contacts",
                    "debug",
                    "friends",
                    "history",
                    "music_files",
                    "music_files_read",
                    "picture_files",
                    "picture_files_read",
                    "video_files",
                    "video_files_read"
                ]
            }
        },
        "1.2": {
            "templates": {
                "common": [
                    "default",
                    "ubuntu-account-plugin",
                    "ubuntu-push-helper",
                    "ubuntu-scope-network",
                    "ubuntu-sdk",
                    "ubuntu-webapp"
                ],
                "reserved": [
                    "unconfined"
                ]
            },
            "policy_groups": {
                "common": [
                    "accounts",
                    "audio",
                    "camera",
                    "connectivity",
                    "content_exchange",
                    "content_exchange_source",
                    "keep-display-on",
                    "location",
                    "microphone",
                    "networking",
                    "push-notification-client",
                    "sensors",
                    "usermetrics",
                    "video",
                    "webview"
                ],
                "reserved": [
                    "calendar",
                    "contacts",
                    "debug",
                    "history",
                    "music_files",
                    "music_files_read",
                    "picture_files",
                    "picture_files_read",
                    "video_files",
                    "video_files_read"
                ]
            }
        },
        "1.3": {
            "templates": {
                "common": [
                    "default",
                    "ubuntu-account-plugin",
                    "ubuntu-push-helper",
                    "ubuntu-scope-network",
                    "ubuntu-sdk",
                    "ubuntu-webapp"
                ],
                "reserved": [
                    "unconfined"
                ]
            },
            "policy_groups": {
                "common": [
                    "accounts",
                    "audio",
                    "camera",
                    "connectivity",
                    "content_exchange",
                    "content_exchange_source",
                    "in-app-purchases",
                    "keep-display-on",
                    "location",
                    "microphone",
                    "networking",
                    "push-notification-client",
                    "sensors",
                    "usermetrics",
                    "video",
                    "webview"
                ],
                "reserved": [
                    "bluetooth",
                    "calendar",
                    "contacts",
                    "debug",
                    "history",
                    "music_files",
                    "music_files_read",
                    "picture_files",
                    "picture_files_read",
                    "video_files",
                    "video_files_read"
                ]
            }
        },
        "15.10": {
            "templates": {
                "common": [
                    "default",
                    "ubuntu-account-plugin",
                    "ubuntu-push-helper",
                    "ubuntu-scope-network",
                    "ubuntu-sdk",
                    "ubuntu-webapp"
                ],
                "reserved": [
                    "unconfined"
                ]
            },
            "policy_groups": {
                "common": [
                    "accounts",
                    "audio",
                    "camera",
                    "connectivity",
                    "content_exchange",
                    "content_exchange_source",
                    "in-app-purchases",
                    "keep-display-on",
                    "location",
                    "microphone",
                    "networking",
                    "push-notification-client",
                    "sensors",
                    "usermetrics",
                    "video",
                    "webview"
                ],
                "reserved": [
                    "bluetooth",
                    "calendar",
                    "contacts",
                    "debug",
                    "history",
                    "music_files",
                    "music_files_read",
                    "picture_files",
                    "picture_files_read",
                    "video_files",
                    "video_files_read"
                ]
            }
        },
        "16.04": {
            "templates": {
                "common": [
                    "default",
                    "ubuntu-account-plugin",
                    "ubuntu-push-helper",
                    "ubuntu-scope-network",
                    "ubuntu-sdk",
                    "ubuntu-webapp"
                ],
                "reserved": [
                    "unconfined"
                ]
            },
            "policy_groups": {
                "common": [
                    "accounts",
                    "audio",
                    "camera",
                    "connectivity",
                    "content_exchange",
                    "content_exchange_source",
                    "fm_radio",
                    "in-app-purchases",
                    "keep-display-on",
                    "location",
                    "microphone",
                    "networking",
                    "push-notification-client",
                    "sensors",
                    "usermetrics",
                    "video",
                    "webview",
                    "nfc"
                ],
                "reserved": [
                    "bluetooth",
                    "calendar",
                    "contacts",
                    "debug",
                    "history",
                    "music_files",
                    "music_files_read",
                    "picture_files",
                    "picture_files_read",
                    "video_files",
                    "video_files_read"
                ]
            }
        },
        "20.04": {
            "templates": {
                "common": [
                    "default",
                    "ubuntu-account-plugin",
                    "ubuntu-push-helper",
                    "ubuntu-scope-network",
                    "ubuntu-sdk",
                    "ubuntu-webapp"
                ],
                "reserved": [
                    "unconfined"
                ]
            },
            "policy_groups": {
                "common": [
                    "accounts",
                    "audio",
                    "camera",
                    "connectivity",
                    "content_exchange",
                    "content_exchange_source",
                    "fm_radio",
                    "in-app-purchases",
                    "keep-display-on",
                    "location",
                    "microphone",
                    "networking",
                    "push-notification-client",
                    "sensors",
                    "usermetrics",
                    "video",
                    "webview",
                    "nfc"
                ],
                "reserved": [
                    "bluetooth",
                    "calendar",
                    "contacts",
                    "debug",
                    "history",
                    "music_files",
                    "music_files_read",
                    "picture_files",
                    "picture_files_read",
                    "video_files",
                    "video_files_read"
                ]
            }
        }
    },
    "ubuntu-snappy": {
        "1.3": {
            "templates": {
                "common": [
                    "default"
                ],
                "reserved": [
                    "unconfined"
                ]
            },
            "policy_groups": {
                "common": [
                    "networking",
                    "network-service",
                    "network-client"
                ],
                "reserved": []
            }
        }
    },
    "ubuntu-core": {
        "15.04": {
            "templates": {
                "common": [
                    "default"
                ],
                "reserved": [
                    "unconfined"
                ]
            },
            "policy_groups": {
                "common": [
                    "network-client",
                    "network-service",
                    "networking"
                ],
                "reserved": [
                    "bluez_client",
                    "docker_client",
                    "network-admin",
                    "network-firewall",
                    "network-status",
                    "snapd"
                ]
            }
        },
        "15.10": {
            "templates": {
                "common": [
                    "default"
                ],
                "reserved": [
                    "unconfined"
                ]
            },
            "policy_groups": {
                "common": [
                    "network-client",
                    "network-service",
                    "networking"
                ],
                "reserved": [
                    "bluez_client",
                    "docker_client",
                    "network-admin",
                    "network-firewall",
                    "network-status",
                    "snapd"
                ]
            }
        },
        "16.04": {
            "templates": {
                "common": [
                    "default"
                ],
                "reserved": []
            },
            "policy_groups": {
                "common": [
                    "bluetooth-control",
                    "bluez",
                    "bool-file",
                    "browser-support",
                    "camera",
                    "content",
                    "cups-control",
                    "docker",
                    "docker-support",
                    "firewall-control",
                    "fuse-support",
                    "fwupd",
                    "gpio",
                    "gsettings",
                    "hardware-observe",
                    "hidraw",
                    "home",
                    "kernel-module-control",
                    "libvirt",
                    "locale-control",
                    "location-control",
                    "location-observe",
                    "log-observe",
                    "lxd-support",
                    "mir",
                    "modem-manager",
                    "mount-observe",
                    "mpris",
                    "network",
                    "network-bind",
                    "network-control",
                    "network-manager",
                    "network-observe",
                    "network-setup-observe",
                    "opengl",
                    "optical-drive",
                    "ppp",
                    "process-control",
                    "pulseaudio",
                    "removable-media",
                    "screen-inhibit-control",
                    "serial-port",
                    "snapd-control",
                    "system-observe",
                    "system-trace",
                    "time-control",
                    "timeserver-control",
                    "timezone-control",
                    "tpm",
                    "udisks2",
                    "upower-observe",
                    "unity7",
                    "x11"
                ],
                "reserved": []
            }
        }
    }
}



'''test_cr_framework.py: tests for the cr_framework module'''
#
# Copyright (C) 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from clickreviews.cr_framework import ClickReviewFramework
import clickreviews.cr_tests as cr_tests


class TestClickReviewFramework(cr_tests.TestClickReview):
    """Tests for the framework review tool."""
    def setUp(self):
        super().setUp()
        self.set_test_pkgfmt("snap", "15.04")

    def test_framework_hook_obsolete(self):
        '''Test check_framework_hook_obsolete()'''
        self.set_test_framework(self.default_appname, "", "")
        c = ClickReviewFramework(self.test_name)

        c.check_framework_hook_obsolete()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_snappy_framework_file_obsolete(self):
        '''Test check_snappy_framework_file_obsolete()'''
        self.set_test_pkg_yaml("type", "framework")
        c = ClickReviewFramework(self.test_name)
        c.check_snappy_framework_file_obsolete()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_snappy_framework_depends(self):
        '''Test check_snappy_framework_depends()'''
        self.set_test_pkg_yaml("type", "framework")
        c = ClickReviewFramework(self.test_name)
        c.check_snappy_framework_depends()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_snappy_framework_depends_bad(self):
        '''Test check_snappy_framework_depends() - bad'''
        self.set_test_pkg_yaml("type", "framework")
        self.set_test_pkg_yaml("frameworks", ['foo'])
        c = ClickReviewFramework(self.test_name)
        c.check_snappy_framework_depends()
        r = c.click_report
        expected_counts = {'info': 0, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_snappy_framework_policy(self):
        '''Test check_snappy_framework_policy()'''
        self.set_test_pkg_yaml("type", "framework")
        c = ClickReviewFramework(self.test_name)
        c.check_snappy_framework_policy()
        r = c.click_report
        expected_counts = {'info': 2, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_snappy_framework_policy_missing(self):
        '''Test check_snappy_framework_policy() - missing'''
        self.set_test_pkg_yaml("type", "framework")
        self.set_test_framework_policy({})
        c = ClickReviewFramework(self.test_name)
        c.check_snappy_framework_policy()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_snappy_framework_policy_unknown(self):
        '''Test check_snappy_framework_policy() - unknown'''
        self.set_test_pkg_yaml("type", "framework")
        self.set_test_framework_policy_unknown(['foo/bar/baz'])
        c = ClickReviewFramework(self.test_name)
        c.check_snappy_framework_policy()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)

    def test_snappy_framework_policy_metadata(self):
        '''Test check_snappy_framework_policy_metadata()'''
        self.set_test_pkg_yaml("type", "framework")
        c = ClickReviewFramework(self.test_name)
        c.check_snappy_framework_policy_metadata()
        r = c.click_report
        expected_counts = {'info': 1, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_snappy_framework_policy_metadata_template(self):
        '''Test check_snappy_framework_policy_metadata() - template missing
           data
        '''
        self.set_test_pkg_yaml("type", "framework")
        tmp = self.test_framework_policy
        tmp['apparmor']['templates']['template-common'] = 'missing data'
        self.set_test_framework_policy(tmp)
        c = ClickReviewFramework(self.test_name)
        c.check_snappy_framework_policy_metadata()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_snappy_framework_policy_metadata_policygroup(self):
        '''Test check_snappy_framework_policy_metadata() - policygroup missing
           data
        '''
        self.set_test_pkg_yaml("type", "framework")
        tmp = self.test_framework_policy
        tmp['seccomp']['policygroups']['policygroup-reserved'] = 'missing data'
        self.set_test_framework_policy(tmp)
        c = ClickReviewFramework(self.test_name)
        c.check_snappy_framework_policy_metadata()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_snappy_framework_policy_matching(self):
        '''Test check_snappy_framework_policy_matching()'''
        self.set_test_pkg_yaml("type", "framework")
        c = ClickReviewFramework(self.test_name)
        c.check_snappy_framework_policy_matching()
        r = c.click_report
        expected_counts = {'info': 8, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_snappy_framework_policy_matching_missing_aa_template(self):
        '''Test check_snappy_framework_policy_matching() - missing aa template
        '''
        self.set_test_pkg_yaml("type", "framework")
        tmp = self.test_framework_policy
        del tmp['apparmor']['templates']['template-common']
        self.set_test_framework_policy(tmp)
        c = ClickReviewFramework(self.test_name)
        c.check_snappy_framework_policy_matching()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_snappy_framework_policy_matching_missing_sc_policygroup(self):
        '''Test check_snappy_framework_policy_matching() - missing sc policy
           group
        '''
        self.set_test_pkg_yaml("type", "framework")
        tmp = self.test_framework_policy
        del tmp['seccomp']['policygroups']['policygroup-reserved']
        self.set_test_framework_policy(tmp)
        c = ClickReviewFramework(self.test_name)
        c.check_snappy_framework_policy_matching()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_snappy_framework_policy_filenames(self):
        '''Test check_snappy_framework_policy_filenames()'''
        self.set_test_pkg_yaml("type", "framework")
        c = ClickReviewFramework(self.test_name)
        c.check_snappy_framework_policy_filenames()
        r = c.click_report
        expected_counts = {'info': 8, 'warn': 0, 'error': 0}
        self.check_results(r, expected_counts)

    def test_snappy_framework_policy_filenames_bad(self):
        '''Test check_snappy_framework_policy_filenames() - bad name'''
        self.set_test_pkg_yaml("type", "framework")
        tmp = self.test_framework_policy
        tmp['seccomp']['policygroups']['policygroup-res_erved'] = "foo"
        self.set_test_framework_policy(tmp)
        c = ClickReviewFramework(self.test_name)
        c.check_snappy_framework_policy_filenames()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 0, 'error': 1}
        self.check_results(r, expected_counts)

    def test_snappy_framework_policy_filenames_bad2(self):
        '''Test check_snappy_framework_policy_filenames() - starts with
           package name
        '''
        self.set_test_pkg_yaml("type", "framework")
        tmp = self.test_framework_policy
        n = self.test_name.split('_')[0]
        tmp['seccomp']['policygroups']['%s-group' % n] = "foo"
        self.set_test_framework_policy(tmp)
        c = ClickReviewFramework(self.test_name)
        c.check_snappy_framework_policy_filenames()
        r = c.click_report
        expected_counts = {'info': None, 'warn': 1, 'error': 0}
        self.check_results(r, expected_counts)



'''overrides.py: overrides for various functions'''
#
# Copyright (C) 2017 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#
# sr_declaration.py overrides
#
# List of attributes that always successfully match (ie, so we don't
# flag on them).
iface_attributes_noflag = ['$SLOT(content)']

#
# sr_lint.py overrides
#
# To be removed. For now we know that snap names have a 1 to 1 mapping
# to publishers so we can whitelist snap names for snap types to not
# flag for manual review.
# NOTE: this will eventually move to assertions
redflagged_snap_types_overrides = {
    'kernel': ['dragonboard-kernel',  # Canonical reference kernels
               'linux-generic-bbb',
               'pc-kernel',
               'pi2-kernel',
               'freescale-ls1043a-kernel',  # @canonical.com kernels
               'hummingboard-kernel',
               'joule-linux',
               'mako-kernel',
               'marvell-armada3700-kernel',
               'nxp-ls1043a-kernel',
               'roseapple-pi-kernel',
               'roseapple-pi-kernel-ondra',
               'artik5-linux',  # 3rd party vendor kernels
               'artik10-linux',
               'bubblegum96-kernel',
               'eragon410-kernel',
               'linux-generic-bbb',
               'rexroth-xm21-kernel',
               'nitrogen-kernel',
               'teal-kernel',
               'telig-kernel',
               'tsimx6-kernel',
               ],
    'os': ['core',
           'ubuntu-core'
           ],
    'gadget': ['dragonboard',  # Canonical reference gadgets
               'pc',
               'pi2',
               'pi3',
               'cm3-ondra',  # @canonical.com gadgets
               'hikey-snappy-gadget',
               'hummingboard',
               'joule',
               'nxp-ls1043ardb-gadget',
               'pi3-ondra',
               'pi3-unipi',
               'pi2kyle',
               'roseapple-pi',
               'sabrelite',
               'wdl-nextcloud',
               'wdl-nextcloud-pi2',
               'artik5',  # 3rd party vendor gadgets
               'artik10',
               'bubblegum96-gadget',
               'dragonboard-turtlebot-kyrofa',
               'eragon410',
               'eragon-sunny',
               'lemaker-guitar-gadget',
               'nitrogen-gadget',
               'pc-turtlebot-kyrofa',
               'rexroth-xm21',
               'subutai-pc',
               'telig',
               'tsimx6-gadget',
               ],
}

# List of snaps that may specify an interface that requires a desktop file but
# not supply a desktop file itself.
desktop_file_exception = ['ffscreencast']

#
# sr_security.py overrides
#
# Files with unusual that we'll allow for certain snaps
sec_mode_overrides = {
    'core': {
        './bin/mount': 'rwsr-xr-x',
        './bin/ping': 'rwsr-xr-x',
        './bin/ping6': 'rwsr-xr-x',
        './bin/su': 'rwsr-xr-x',
        './bin/umount': 'rwsr-xr-x',
        './etc/chatscripts': 'rwxr-s---',
        './etc/ppp/peers': 'rwxr-s---',
        './run/lock': 'rwxrwxrwt',
        './sbin/pam_extrausers_chkpwd': 'rwxr-sr-x',
        './sbin/unix_chkpwd': 'rwxr-sr-x',
        './tmp': 'rwxrwxrwt',
        './usr/bin/chage': 'rwxr-sr-x',
        './usr/bin/chfn': 'rwsr-xr-x',
        './usr/bin/chsh': 'rwsr-xr-x',
        './usr/bin/crontab': 'rwxr-sr-x',
        './usr/bin/dotlockfile': 'rwxr-sr-x',
        './usr/bin/expiry': 'rwxr-sr-x',
        './usr/bin/gpasswd': 'rwsr-xr-x',
        './usr/bin/mail-lock': 'rwxr-sr-x',
        './usr/bin/mail-unlock': 'rwxr-sr-x',
        './usr/bin/mail-touchlock': 'rwxr-sr-x',
        './usr/bin/newgrp': 'rwsr-xr-x',
        './usr/bin/passwd': 'rwsr-xr-x',
        './usr/bin/ssh-agent': 'rwxr-sr-x',
        './usr/bin/sudo': 'rwsr-xr-x',
        './usr/bin/wall': 'rwxr-sr-x',
        './usr/lib/dbus-1.0/dbus-daemon-launch-helper': 'rwsr-xr--',
        './usr/lib/openssh/ssh-keysign': 'rwsr-xr-x',
        './usr/lib/snapd/snap-confine': 'rwsr-xr-x',
        './usr/local/lib/python3.5': 'rwxrwsr-x',
        './usr/local/lib/python3.5/dist-packages': 'rwxrwsr-x',
        './usr/sbin/pppd': 'rwsr-xr--',
        './var/local': 'rwxrwsr-x',
        './var/mail': 'rwxrwsr-x',
        './var/spool/cron/crontabs': 'rwx-wx--T',
        './var/tmp': 'rwxrwxrwt',
    },
    'chrome-test': {  # chrome-test from Canonical
        './opt/google/chrome/chrome-sandbox': 'rwsr-xr-x',
    },
    'openwrt': {  # demo from Canonical
        './rootfs/tmp': 'rwxrwxrwt',
    },
    'ubuntu-core': {
        './bin/mount': 'rwsr-xr-x',
        './bin/ping': 'rwsr-xr-x',
        './bin/ping6': 'rwsr-xr-x',
        './bin/su': 'rwsr-xr-x',
        './bin/umount': 'rwsr-xr-x',
        './etc/chatscripts': 'rwxr-s---',
        './etc/ppp/peers': 'rwxr-s---',
        './run/lock': 'rwxrwxrwt',
        './sbin/pam_extrausers_chkpwd': 'rwxr-sr-x',
        './sbin/unix_chkpwd': 'rwxr-sr-x',
        './tmp': 'rwxrwxrwt',
        './usr/bin/chage': 'rwxr-sr-x',
        './usr/bin/chfn': 'rwsr-xr-x',
        './usr/bin/chsh': 'rwsr-xr-x',
        './usr/bin/crontab': 'rwxr-sr-x',
        './usr/bin/dotlockfile': 'rwxr-sr-x',
        './usr/bin/expiry': 'rwxr-sr-x',
        './usr/bin/gpasswd': 'rwsr-xr-x',
        './usr/bin/mail-lock': 'rwxr-sr-x',
        './usr/bin/mail-unlock': 'rwxr-sr-x',
        './usr/bin/mail-touchlock': 'rwxr-sr-x',
        './usr/bin/newgrp': 'rwsr-xr-x',
        './usr/bin/passwd': 'rwsr-xr-x',
        './usr/bin/ssh-agent': 'rwxr-sr-x',
        './usr/bin/sudo': 'rwsr-xr-x',
        './usr/bin/wall': 'rwxr-sr-x',
        './usr/lib/dbus-1.0/dbus-daemon-launch-helper': 'rwsr-xr--',
        './usr/lib/openssh/ssh-keysign': 'rwsr-xr-x',
        './usr/lib/snapd/snap-confine': 'rwsr-xr-x',
        './usr/local/lib/python3.5': 'rwxrwsr-x',
        './usr/local/lib/python3.5/dist-packages': 'rwxrwsr-x',
        './usr/sbin/pppd': 'rwsr-xr--',
        './var/local': 'rwxrwsr-x',
        './var/mail': 'rwxrwsr-x',
        './var/spool/cron/crontabs': 'rwx-wx--T',
        './var/tmp': 'rwxrwxrwt',
    },
}

# Snaps that may specify 'daemon' with 'browser-support'. Normally this
# shouldn't be granted because snaps running with 'daemon' run as root and this
# grants more privileges than intended.
sec_browser_support_overrides = ['screencloudplayer',
                                 'webdemo']
